package repository

import (
	"context"
	"fmt"
	"testing"
	"time"

	"code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/entity"
	"code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/repository"
	mgo "code.30cm.net/digimon/library-go/mongo"
	"github.com/alicebob/miniredis/v2"
	"github.com/stretchr/testify/assert"
	"github.com/zeromicro/go-zero/core/stores/cache"
	"github.com/zeromicro/go-zero/core/stores/mon"
	"github.com/zeromicro/go-zero/core/stores/redis"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func SetupTestCategoryRepository(db string) (repository.CategoryRepository, func(), error) {
	h, p, tearDown, err := startMongoContainer()
	if err != nil {
		return nil, nil, err
	}
	s, _ := miniredis.Run()

	conf := &mgo.Conf{
		Schema:                           Schema,
		Host:                             fmt.Sprintf("%s:%s", h, p),
		Database:                         db,
		MaxStaleness:                     300,
		MaxPoolSize:                      100,
		MinPoolSize:                      100,
		MaxConnIdleTime:                  300,
		Compressors:                      []string{},
		EnableStandardReadWriteSplitMode: false,
		ConnectTimeoutMs:                 3000,
	}

	cacheConf := cache.CacheConf{
		cache.NodeConf{
			RedisConf: redis.RedisConf{
				Host: s.Addr(),
				Type: redis.NodeType,
			},
			Weight: 100,
		},
	}

	cacheOpts := []cache.Option{
		cache.WithExpiry(1000 * time.Microsecond),
		cache.WithNotFoundExpiry(1000 * time.Microsecond),
	}

	param := CategoryRepositoryParam{
		Conf:      conf,
		CacheConf: cacheConf,
		CacheOpts: cacheOpts,
		DBOpts: []mon.Option{
			mgo.SetCustomDecimalType(),
			mgo.InitMongoOptions(*conf),
		},
	}

	repo := MustCategoryRepository(param)
	//_, _ = repo.Index20250317001UP(context.Background())

	return repo, tearDown, nil
}

func TestCategoryRepository_Insert(t *testing.T) {
	repo, tearDown, err := SetupTestCategoryRepository("testDB")
	defer tearDown()
	assert.NoError(t, err)

	ctx := context.Background()
	testData := &entity.Category{
		Name: "Test Category",
	}

	err = repo.Insert(ctx, testData)
	assert.NoError(t, err, "插入操作應該成功")
	assert.NotZero(t, testData.ID, "ID 應該被生成")
	assert.NotNil(t, testData.CreatedAt, "CreateAt 應該被設置")
	assert.NotNil(t, testData.UpdatedAt, "UpdateAt 應該被設置")
}

func TestCategoryRepository_FindOneByID(t *testing.T) {
	repo, tearDown, err := SetupTestCategoryRepository("testDB")
	defer tearDown()
	assert.NoError(t, err)

	ctx := context.Background()
	testData := &entity.Category{
		Name: "Test Category Find",
	}
	_ = repo.Insert(ctx, testData)

	tests := []struct {
		name        string
		id          string
		expectError bool
	}{
		{
			name:        "Valid ID - Find Record",
			id:          testData.ID.Hex(),
			expectError: false,
		},
		{
			name:        "Invalid ID Format",
			id:          "invalid_id",
			expectError: true,
		},
		{
			name:        "Nonexistent ID",
			id:          primitive.NewObjectID().Hex(),
			expectError: true,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			result, err := repo.FindOneByID(ctx, tt.id)
			if tt.expectError {
				assert.Error(t, err)
				assert.Nil(t, result)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, testData.Name, result.Name)
			}
		})
	}
}

func TestCategoryRepository_Update(t *testing.T) {
	repo, tearDown, err := SetupTestCategoryRepository("testDB")
	defer tearDown()
	assert.NoError(t, err)

	ctx := context.Background()
	testData := &entity.Category{
		Name: "Test Category Update",
	}
	_ = repo.Insert(ctx, testData)
	updatedData := &entity.Category{
		ID:   testData.ID,
		Name: "Updated Category Name",
	}
	result, err := repo.Update(ctx, testData.ID.Hex(), updatedData)
	assert.NoError(t, err, "更新操作應該成功")
	assert.Equal(t, int64(1), result.ModifiedCount, "更新的文件數量應為 1")

	// 檢查更新後的數據
	updatedRecord, err := repo.FindOneByID(ctx, testData.ID.Hex())
	assert.NoError(t, err, "應該可以找到更新的記錄")
	assert.Equal(t, "Updated Category Name", updatedRecord.Name, "Category 名稱應該被更新")
}

func TestCategoryRepository_Delete(t *testing.T) {
	repo, tearDown, err := SetupTestCategoryRepository("testDB")
	defer tearDown()
	assert.NoError(t, err)

	ctx := context.Background()
	testData := &entity.Category{
		Name: "Test Category Delete",
	}
	_ = repo.Insert(ctx, testData)

	tests := []struct {
		name        string
		id          string
		expectError bool
		expectCount int64
	}{
		{
			name:        "Valid ID - Successful Deletion",
			id:          testData.ID.Hex(),
			expectError: false,
			expectCount: 1,
		},
		{
			name:        "Invalid ID Format",
			id:          "invalid_id",
			expectError: true,
			expectCount: 0,
		},
		{
			name:        "Nonexistent ID",
			id:          primitive.NewObjectID().Hex(),
			expectError: false,
			expectCount: 0,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			count, err := repo.Delete(ctx, tt.id)
			if tt.expectError {
				assert.Error(t, err)
				assert.Equal(t, int64(0), count)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, tt.expectCount, count)
			}
		})
	}
}

func TestCategoryRepository_ListProduct(t *testing.T) {
	repo, tearDown, err := SetupTestCategoryRepository("testDB")
	defer tearDown()
	assert.NoError(t, err)

	ctx := context.Background()
	// 插入測試數據
	for i := 0; i < 5; i++ {
		_ = repo.Insert(ctx, &entity.Category{
			Name: "Test Category List",
		})
	}

	params := &repository.CategoryQueryParams{
		PageSize:  2,
		PageIndex: 1,
	}

	categories, count, err := repo.ListCategory(ctx, params)
	assert.NoError(t, err, "查詢應該成功")
	assert.Equal(t, int64(5), count, "總數應該為 5")
	assert.Len(t, categories, int(params.PageSize), "返回的數據應符合頁面大小")
}

func TestCategoryRepository_IsCategoryExists(t *testing.T) {
	repo, tearDown, err := SetupTestCategoryRepository("testDB")
	defer tearDown()
	assert.NoError(t, err)

	ctx := context.Background()

	// 插入一些測試分類數據
	category1 := &entity.Category{ID: primitive.NewObjectID()}
	category2 := &entity.Category{ID: primitive.NewObjectID()}
	category3 := &entity.Category{ID: primitive.NewObjectID()}
	_ = repo.Insert(ctx, category1)
	_ = repo.Insert(ctx, category2)

	// 定義測試案例
	tests := []struct {
		name        string
		ids         []string
		expectedIDs []string
	}{
		{
			name:        "部分分類存在",
			ids:         []string{category1.ID.Hex(), category2.ID.Hex(), category3.ID.Hex()}, // category3 不存在
			expectedIDs: []string{category1.ID.Hex(), category2.ID.Hex()},
		},
		{
			name:        "所有分類存在",
			ids:         []string{category1.ID.Hex(), category2.ID.Hex()},
			expectedIDs: []string{category1.ID.Hex(), category2.ID.Hex()},
		},
		{
			name:        "無分類存在",
			ids:         []string{category3.ID.Hex()}, // category3 不存在
			expectedIDs: []string{},
		},
		{
			name:        "包含無效 ID",
			ids:         []string{category1.ID.Hex(), "invalid_id"}, // "invalid_id" 是無效 ID 格式
			expectedIDs: []string{category1.ID.Hex()},
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			result := repo.IsCategoryExists(ctx, tt.ids)

			assert.ElementsMatch(t, tt.expectedIDs, result, "返回的存在ID應該與期望值匹配")
		})
	}
}
